package com.company.audiodemo.view.audiodemostreaminternalcntls;


import com.company.audiodemo.view.audiodemo.component.Audio;
import com.company.audiodemo.view.main.MainView;

import com.vaadin.flow.component.AbstractField;
import com.vaadin.flow.component.notification.NotificationVariant;
import com.vaadin.flow.component.upload.Receiver;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.StreamRegistration;
import com.vaadin.flow.server.StreamResource;
import com.vaadin.flow.server.VaadinSession;
import io.jmix.core.FileRef;
import io.jmix.core.FileStorage;
import io.jmix.core.FileStorageException;
import io.jmix.flowui.Notifications;
import io.jmix.flowui.component.upload.FileStorageUploadField;
import io.jmix.flowui.component.upload.receiver.FileTemporaryStorageBuffer;
import io.jmix.flowui.kit.component.upload.event.FileUploadSucceededEvent;
import io.jmix.flowui.upload.TemporaryStorage;
import io.jmix.flowui.view.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.*;
import java.util.UUID;

@Route(value = "audio-demo-stream-internal-cntls-view", layout = MainView.class)
@ViewController("demo_AudioDemoStreamInternalCntlsView")
@ViewDescriptor("audio-demo-stream-internal-cntls-view.xml")
public class AudioDemoStreamInternalCntlsView extends StandardView {

    private VaadinSession vaadinSession;
    private StreamRegistration registration = null;
    private FileRef audioFileRef = null;
    private StreamResource audioStreamResource;
    @ViewComponent
    private Audio myAudio;
    @ViewComponent
    private FileStorageUploadField audioFileStorageUploadField;
    @Autowired
    private Notifications notifications;
    @Autowired
    private FileStorage fileStorage;
    @Autowired
    private TemporaryStorage temporaryStorage;
    private final Logger log = LoggerFactory.getLogger(AudioDemoStreamInternalCntlsView.class);

    @Subscribe
    public void onInit(final InitEvent event) {
        vaadinSession = VaadinSession.getCurrent();
    }

    @Subscribe
    public void onBeforeShow(final BeforeShowEvent event) {
        audioFileStorageUploadField.setMaxFileSize(100000000);
    }

    @Subscribe("audioFileStorageUploadField")
    public void onAudioFileStorageUploadFieldFileUploadSucceeded(final FileUploadSucceededEvent<FileStorageUploadField> event) {
        UUID fileId;
        File file;

        Receiver receiver = event.getReceiver();
        if (receiver instanceof FileTemporaryStorageBuffer storageBuffer) {
            fileId = storageBuffer.getFileData().getFileInfo().getId();
            file = temporaryStorage.getFile(fileId);
        } else {
            fileId = null;
            file = null;
        }

        if (file != null) {
            notifications.create("Audio file stored successfully - " + file.getAbsolutePath())
                    .withType(Notifications.Type.DEFAULT)
                    .withThemeVariant(NotificationVariant.LUMO_SUCCESS)
                    .show();
        } else {
            String failedMessage = "Audio file storage failed - " + event.getFileName();
            notifications.create(failedMessage)
                    .withType(Notifications.Type.ERROR)
                    .withThemeVariant(NotificationVariant.LUMO_ERROR)
                    .show();
            log.error(failedMessage);
            return;
        }

        if (audioFileRef != null) {   // remove the last uploaded audio file during this session, since a new one was chosen
            fileStorage.removeFile(audioFileRef);
            audioFileRef = null;
        }

        FileRef audioFileRef = temporaryStorage.putFileIntoStorage(fileId, event.getFileName());
        audioFileStorageUploadField.setValue(audioFileRef);
        audioFileStorageUploadField.setClearButtonVisible(true);

        audioStreamResource = new StreamResource(event.getFileName(), () -> {
            byte[] bytes = new byte[0];
            try {
                bytes = fileStorage.openStream(audioFileRef).readAllBytes();
            } catch (IOException e) {
                log.error("Unable to load audio file - " + event.getFileName() + " IOException: " + e.getMessage());
            }
            return new ByteArrayInputStream(bytes);
        }).setContentType(event.getMimeType());
//       }).setContentType("audio/mp3");

        registration = vaadinSession.getResourceRegistry().registerResource(audioStreamResource);
        audioStreamResource.setCacheTime(5000L);
        myAudio.setSource(audioStreamResource);
        myAudio.setText(audioFileRef.getFileName());
    }

    @Subscribe("audioFileStorageUploadField")
    public void onAudioFileStorageUploadFieldComponentValueChange(final AbstractField.ComponentValueChangeEvent<FileStorageUploadField, ?> event) {

        if (event.getValue() == null) { // clear audioFileStorageUploadField
            audioFileStorageUploadField.setClearButtonVisible(false);
            myAudio.pause();
            myAudio.setSource((String) null);
            if (registration != null) {
                registration.unregister();
                registration = null;
            }
            try {
                if (audioFileRef != null) {   // remove the last uploaded audio file for this session
                    fileStorage.removeFile(audioFileRef);
                    audioFileRef = null;
                }
            } catch (FileStorageException e) {
                log.error("Unable to remove the previous audio file - " + audioFileRef.getFileName());
            }
        }
    }

    @Subscribe
    public void onAfterClose(final AfterCloseEvent event) {

        if (registration != null) {
            registration.unregister();
        }

        try {
            if (audioFileRef != null) {
                fileStorage.removeFile(audioFileRef);
            }
        } catch (FileStorageException e) {
            log.error("Unable to remove audio file " + audioFileRef.getFileName());
        }
    }
}